﻿namespace ApNegociosSL.LoginUI
{
  using System;
  using System.ComponentModel.DataAnnotations;
  using System.ServiceModel.DomainServices.Client;
  using System.ServiceModel.DomainServices.Client.ApplicationServices;
  using ApNegociosSL.Web.Resources;

  /// <summary>
  /// Esta entidad interna se utiliza para facilitar el enlace entre los controles de la IU (DataForm y la etiqueta que muestra un error de validación) y las credenciales de inicio de sesión especificadas por el usuario.
  /// </summary>
  public class LoginInfo : ComplexObject
  {
    private string userName;
    private bool rememberMe;
    private LoginOperation currentLoginOperation;

    /// <summary>
    /// Obtiene y establece el nombre de usuario.
    /// </summary>
    [Display(Name = "UserNameLabel", ResourceType = typeof(RegistrationDataResources))]
    [Required]
    public string UserName
    {
      get
      {
        return this.userName;
      }

      set
      {
        if (this.userName != value)
        {
          this.ValidateProperty("UserName", value);
          this.userName = value;
          this.RaisePropertyChanged("UserName");
        }
      }
    }

    /// <summary>
    /// Obtiene o establece una función que devuelve la contraseña.
    /// </summary>
    internal Func<string> PasswordAccessor { get; set; }

    /// <summary>
    /// Obtiene y establece la contraseña.
    /// </summary>
    [Display(Name = "PasswordLabel", ResourceType = typeof(RegistrationDataResources))]
    [Required]
    public string Password
    {
      get
      {
        return (this.PasswordAccessor == null) ? string.Empty : this.PasswordAccessor();
      }
      set
      {
        this.ValidateProperty("Password", value);

        // No almacene la contraseña en un campo privado: no se debe almacenar en memoria en texto sin formato.
        // En su lugar, el PasswordAccessor proporcionado sirve como almacén de copia de seguridad del valor.

        this.RaisePropertyChanged("Password");
      }
    }

    /// <summary>
    /// Obtiene y establece el valor que indica si se debe registrar la información de autenticación del usuario para futuros inicios de sesión.
    /// </summary>
    [Display(Name = "RememberMeLabel", ResourceType = typeof(ApplicationStrings))]
    public bool RememberMe
    {
      get
      {
        return this.rememberMe;
      }

      set
      {
        if (this.rememberMe != value)
        {
          this.ValidateProperty("RememberMe", value);
          this.rememberMe = value;
          this.RaisePropertyChanged("RememberMe");
        }
      }
    }

    /// <summary>
    /// Obtiene o establece la operación de inicio de sesión actual.
    /// </summary>
    internal LoginOperation CurrentLoginOperation
    {
      get
      {
        return this.currentLoginOperation;
      }
      set
      {
        if (this.currentLoginOperation != value)
        {
          if (this.currentLoginOperation != null)
          {
            this.currentLoginOperation.Completed -= (s, e) => this.CurrentLoginOperationChanged();
          }

          this.currentLoginOperation = value;

          if (this.currentLoginOperation != null)
          {
            this.currentLoginOperation.Completed += (s, e) => this.CurrentLoginOperationChanged();
          }

          this.CurrentLoginOperationChanged();
        }
      }
    }

    /// <summary>
    /// Obtiene un valor que indica si el usuario está iniciando sesión en ese momento.
    /// </summary>
    [Display(AutoGenerateField = false)]
    public bool IsLoggingIn
    {
      get
      {
        return this.CurrentLoginOperation != null && !this.CurrentLoginOperation.IsComplete;
      }
    }

    /// <summary>
    /// Obtiene un valor que indica si el usuario puede iniciar sesión en ese momento.
    /// </summary>
    [Display(AutoGenerateField = false)]
    public bool CanLogIn
    {
      get
      {
        return !this.IsLoggingIn;
      }
    }

    /// <summary>
    /// Genera notificaciones de cambio de propiedades relacionadas con la operación cuando cambia la operación actual de inicio de sesión.
    /// </summary>
    private void CurrentLoginOperationChanged()
    {
      this.RaisePropertyChanged("IsLoggingIn");
      this.RaisePropertyChanged("CanLogIn");
    }

    /// <summary>
    /// Crea un nueva instancia de <see cref="LoginParameters"/> con los datos almacenados en esta entidad.
    /// </summary>
    public LoginParameters ToLoginParameters()
    {
      return new LoginParameters(this.UserName, this.Password, this.RememberMe, null);
    }
  }
}
